import Debug from 'debug'
import { ErrorDescEnum, HttpError } from '../classes/HttpError'
import { PrismaClient } from '@prisma/client'

console.log = Debug('invoiceIssuer:func/createPayer.ts')

/**
 * POST /payer
 * @summary 创建一个新的付款人。这个付款人与收据上标识的收款人信息分离，但内部数据库关联（以便检索）。
 * @param {string} name - 付款人的名字
 * @param {string} address - 付款人的地址
 * @param {string} email - 付款人的电子邮件地址
 * @param {string} abn - 付款人的澳大利亚商业号码
 */
export default async (prisma: PrismaClient, name: string, address: string[], email: string, abn?: string) => {
	// 验证 ABN 是否已存在
	const existingPayer = await prisma.payer.findFirst({
		where: {
			payer_abn: abn
		}
	})
	if (existingPayer) throw new HttpError(ErrorDescEnum.item_exists, 400, ['abn'])

	// 创建新的付款人
	const payer = await prisma.payer.create({
		data: {
			payer_name: name,
			payer_address: address.join('\n'),
			payer_email: email,
			payer_abn: abn
		}
	})
	return payer
}