import Debug from 'debug'
import { ErrorDescEnum, HttpError } from '../classes/HttpError'
import { PrismaClient } from '@prisma/client'
import InvoiceItem from '../classes/InvoiceItem'
import Invoice from '../classes/Invoice'

console.log = Debug('invoiceIssuer:func/issueInvoice.ts')

/**
 * POST /invoice
 * @summary 创建一个新的收据
 * @param {number} payerId - 付款人的 ID
 * @param {Date[]} period - 收据的账单周期
 * @param {InvoiceItem[]} items - 收据的项目列表
 * @return {string} - 新收据的 ID
 */
export default async (prisma: PrismaClient, payerId: number, period: Date[], items: InvoiceItem[], dueDate: Date, note?: string) => {
	// 验证付款人是否存在
	const payer = await prisma.payer.findUnique({
		where: {
			payer_id: payerId
		}
	})
	if (!payer) throw new HttpError(ErrorDescEnum.related_item_not_found, 400, ['payer_id'])

	// 创建新收据
	// 获取当天最大收据后缀
	const today = new Date()
	const todayStart = new Date(today.getFullYear(), today.getMonth(), today.getDate())
	const todayEnd = new Date(today.getFullYear(), today.getMonth(), today.getDate() + 1)
	const maxSuffix = await prisma.invoice.findFirst({
		where: {
			AND: [
				{ invoice_date: { gte: todayStart } },
				{ invoice_date: { lt: todayEnd } }
			]
		},
		orderBy: {
			invoice_suffix_code: 'desc'
		}
	})

	// 创建收据实现
	const newInvoice = new Invoice(
		maxSuffix ? maxSuffix.invoice_suffix_code + 1 : 1,
		today,
		period,
		items,
		dueDate,
		{
			id: payer.payer_id,
			name: payer.payer_name,
			address: payer.payer_address,
			abn: payer.payer_abn ? payer.payer_abn : undefined,
			email: payer.payer_email
		}
	)
}